\ stepper.frt
\ chu 16:09 29.09.2010
\
\ ==============================================================
\
\ Stepper motor driver
\

\ The normal sequence 
\ 0 1 0 1
\ 1 0 0 1
\ 1 0 1 0 
\ 0 1 1 0

\ The wave sequence
\ 0 0 0 1
\ 1 0 0 0
\ 0 0 1 0
\ 0 1 0 0

\ The half-step sequence
\ 0 1 0 1	\ 0x05
\ 0 0 0 1	\ 0x01
\ 1 0 0 1	\ 0x09
\ 1 0 0 0	\ 0x08
\ 1 0 1 0	\ 0x0A
\ 0 0 1 0	\ 0x02
\ 0 1 1 0	\ 0x06
\ 0 1 0 0	\ 0x04

\
\ ==============================================================
\

  marker -stepper-

  base @ hex
  create phase
	 55 ,
	 11 ,
	 99 ,
	 88 ,
	 AA ,
	 22 ,
	 66 ,
	 44 ,
  base !

  1 constant half-step
  2 constant wave
  2 constant normal 

\
\ ==============================================================
\

  : -- create over , + does> i@ + ;
  
  0
  1 -- idx		\ displacement in phase table
  1 -- sequence		\ value to change idx
  1 -- shft		\ ***reserved
  1 -- ststate		\ motor status
  1 cells -- dly	\ before next step
  1 cells -- direction	\ boolean value
  1 cells -- steps	\ to rotate
  1 cells -- total	\ steps, signed
  1 cells -- res1	\ ***reserved
  1 cells -- motor	\ pinmask:portadr
  constant  /stepper

\
\ ==============================================================
\ example:create new stepper motor structure with name 'altitude'
\ 	  PORTA F stepper: altitude

  : motor!    ( portaddr pinmask stm_addr -- )
	motor swap over c! 1+ c!
	;

  : stepper:  ( portaddr pinmask -- ) 
	here /stepper allot dup constant motor!
	;

  : motor@  ( stm_addr -- pinmask portaddr )
	motor dup c@ swap 1+ c@
	;

\
\ example:
\ 5 half-step altitude init_stm!

  : init_stm!  ( dly sequence stm_addr -- )
	swap over  sequence c!
	swap over       dly  !
	   0 over       idx c!
	   0 over direction  !
	   0 over     steps  !
	   0 over     total  !
	   0 over   ststate c!
	   0 over      res1  !
	   0 over      shft c!
	   motor@   pin_output
	;

  : idx+    ( stm_addr --  )
	dup  idx c@
	over sequence c@ + 7 and
	swap idx c!
	;

  : idx-    ( stm_addr --  )
	dup  idx c@
	over sequence c@ - 7 and
	swap idx c!
	;

  : total+  ( stm_addr --  )
	dup  total @
	over sequence c@ +
	swap total !
	;

  : total-  ( stm_addr --  )
	dup  total @
	over sequence c@ -
	swap total !
	;

  : step    ( stm_addr --  )
	dup direction @     if
	dup idx- dup total- else 
	dup idx+ dup total+ then 
	phase over idx c@ + 
	i@ swap motor@ pin!
	;

  : rotate  ( stm_addr n --  )
	0 ?do dup step pause dup dly @ ms loop drop 
	;

  : direction!  ( stm_a f -- stm_a ) over direction ! ;

  : reverse ( stm_addr --  )
	direction dup @ invert swap ! 
	;

  : half!   ( stm_addr --  )
	sequence half-step swap c!
	;

  : normal! ( stm_addr --  )
	dup half! dup idx c@ 1 and if dup step then 
	sequence normal swap c!
	;

  : wave!   ( stm_addr --  )
	dup half! dup idx c@ 1 and 0= if dup step then 
	sequence wave swap c!
	;

\    
  : stopoff ( stm_addr -- )
	0 swap motor@ pin!
	;

\    "" 
  : park    ( stm_addr -- )
	dup
	total @ dup 0< if
	  0 direction! else
	 -1 direction! then
	abs rotate
	;


\
\ ==============================================================
\ inhibit steps update

  $80 constant lock_stm

  : stm_lock?  ( stm_addr -- f )  
	ststate c@ lock_stm and 
	;

  : stm_lock   ( stm_addr -- )
	ststate dup c@ lock_stm or swap c! 
	;

  : stm_unlock ( stm_addr -- )
	ststate dup c@ lock_stm invert and swap c!
	;

\
\ ==============================================================
\


  : stm?    ( stm_addr --  )
	cr base @ >r 
	hex
	dup motor@
	." port............." u. cr
	." mask............." u. cr
	." ................." cr
	decimal
	." idx.............." dup idx      c@ u. cr
	." sequence........." dup sequence c@ u. cr
	." delay............" dup dly       @ u. cr
	." direction........" dup direction @  . cr
	." steps to rotate.." dup steps     @  . cr
	." total steps......" dup total     @  . cr
	." motor status....."     ststate  c@ .x cr

	r> base !
	;

\
\ ==============================================================
\
